@@ -236,13 +236,21 @@ pub async fn remote_context() -> SessionContext {
 
 #[allow(dead_code)]
 pub async fn standalone_context_with_state() -> SessionContext {
-    let state = SessionStateBuilder::new().with_default_features().build();
+    let config = SessionConfig::new_with_ballista();
+    let state = SessionStateBuilder::new()
+        .with_config(config)
+        .with_default_features()
+        .build();
     SessionContext::standalone_with_state(state).await.unwrap()
 }
 
 #[allow(dead_code)]
 pub async fn remote_context_with_state() -> SessionContext {
-    let state = SessionStateBuilder::new().with_default_features().build();
+    let config = SessionConfig::new_with_ballista();
+    let state = SessionStateBuilder::new()
+        .with_config(config)
+        .with_default_features()
+        .build();
     let (host, port) = setup_test_cluster_with_state(state.clone()).await;
     SessionContext::remote_with_state(&format!("df://{host}:{port}"), state)
         .await
