@@ -37,7 +37,7 @@ use datafusion::arrow::error::ArrowError;
 use datafusion::arrow::record_batch::RecordBatch;
 use datafusion::common::runtime::SpawnedTask;
 
-use datafusion::error::Result;
+use datafusion::error::{DataFusionError, Result};
 use datafusion::physical_plan::metrics::{ExecutionPlanMetricsSet, MetricsSet};
 use datafusion::physical_plan::{
     ColumnStatistics, DisplayAs, DisplayFormatType, ExecutionPlan, Partitioning,
@@ -49,7 +49,7 @@ use crate::error::BallistaError;
 use datafusion::execution::context::TaskContext;
 use datafusion::physical_plan::stream::RecordBatchStreamAdapter;
 use itertools::Itertools;
-use log::{debug, error};
+use log::{debug, error, trace};
 use rand::prelude::SliceRandom;
 use rand::rng;
 use tokio::sync::{mpsc, Semaphore};
@@ -101,10 +101,14 @@ impl DisplayAs for ShuffleReaderExec {
     ) -> std::fmt::Result {
         match t {
             DisplayFormatType::Default | DisplayFormatType::Verbose => {
-                write!(f, "ShuffleReaderExec: partitions={}", self.partition.len())
+                write!(
+                    f,
+                    "ShuffleReaderExec: partitioning={:?}",
+                    self.properties.partitioning,
+                )
             }
             DisplayFormatType::TreeRender => {
-                write!(f, "partitions={}", self.partition.len())
+                write!(f, "partitioning={:?}", self.properties.partitioning)
             }
         }
     }
@@ -132,14 +136,15 @@ impl ExecutionPlan for ShuffleReaderExec {
 
     fn with_new_children(
         self: Arc<Self>,
-        _children: Vec<Arc<dyn ExecutionPlan>>,
+        children: Vec<Arc<dyn ExecutionPlan>>,
     ) -> Result<Arc<dyn ExecutionPlan>> {
-        Ok(Arc::new(ShuffleReaderExec::try_new(
-            self.stage_id,
-            self.partition.clone(),
-            self.schema.clone(),
-            self.properties().output_partitioning().clone(),
-        )?))
+        if children.is_empty() {
+            Ok(self)
+        } else {
+            Err(DataFusionError::Plan(
+                "Ballista ShuffleReaderExec does not support children plans".to_owned(),
+            ))
+        }
     }
 
     fn execute(
@@ -155,6 +160,15 @@ impl ExecutionPlan for ShuffleReaderExec {
         let max_request_num =
             config.ballista_shuffle_reader_maximum_concurrent_requests();
         let max_message_size = config.ballista_grpc_client_max_message_size();
+        let force_remote_read = config.ballista_shuffle_reader_force_remote_read();
+        let prefer_flight = config.ballista_shuffle_reader_remote_prefer_flight();
+
+        if force_remote_read {
+            debug!(
+            "All shuffle partitions will be read as remote partitions! To disable this behavior set: `{}=false`",
+            crate::config::BALLISTA_SHUFFLE_READER_FORCE_REMOTE_READ
+        );
+        }
 
         log::debug!(
             "ShuffleReaderExec::execute({task_id}) max_request_num: {max_request_num}, max_message_size: {max_message_size}"
@@ -175,9 +189,13 @@ impl ExecutionPlan for ShuffleReaderExec {
             .collect();
         // Shuffle partitions for evenly send fetching partition requests to avoid hot executors within multiple tasks
         partition_locations.shuffle(&mut rng());
-
-        let response_receiver =
-            send_fetch_partitions(partition_locations, max_request_num, max_message_size);
+        let response_receiver = send_fetch_partitions(
+            partition_locations,
+            max_request_num,
+            max_message_size,
+            force_remote_read,
+            prefer_flight,
+        );
 
         let result = RecordBatchStreamAdapter::new(
             Arc::new(self.schema.as_ref().clone()),
@@ -190,17 +208,75 @@ impl ExecutionPlan for ShuffleReaderExec {
         Some(self.metrics.clone_inner())
     }
 
-    fn statistics(&self) -> Result<Statistics> {
-        Ok(stats_for_partitions(
-            self.schema.fields().len(),
-            self.partition
-                .iter()
-                .flatten()
-                .map(|loc| loc.partition_stats),
-        ))
+    fn partition_statistics(&self, partition: Option<usize>) -> Result<Statistics> {
+        if let Some(idx) = partition {
+            let partition_count = self.properties().partitioning.partition_count();
+            if idx >= partition_count {
+                return datafusion::common::internal_err!(
+                    "Invalid partition index: {}, the partition count is {}",
+                    idx,
+                    partition_count
+                );
+            }
+            let stat_for_partition =
+                stats_for_partition(idx, self.schema.fields().len(), &self.partition);
+
+            trace!(
+                "shuffle reader at stage: {} and partition {} returned statistics: {:?}",
+                self.stage_id,
+                idx,
+                stat_for_partition
+            );
+            stat_for_partition
+        } else {
+            let stats_for_partitions = stats_for_partitions(
+                self.schema.fields().len(),
+                self.partition
+                    .iter()
+                    .flatten()
+                    .map(|loc| loc.partition_stats),
+            );
+            trace!("shuffle reader at stage: {} returned statistics for all partitions: {:?}", self.stage_id, stats_for_partitions);
+            Ok(stats_for_partitions)
+        }
     }
 }
 
+fn stats_for_partition(
+    partition: usize,
+    num_fields: usize,
+    partition_locations: &[Vec<PartitionLocation>],
+) -> Result<Statistics> {
+    // TODO stats: add column statistics to PartitionStats
+    let (num_rows, total_byte_size) = partition_locations
+        .iter()
+        .map(|location| {
+            // extract requested partitions
+            location
+                .get(partition)
+                .map(|p| p.partition_stats)
+                .map(|p| (p.num_rows, p.num_bytes))
+                .unwrap_or_default()
+        })
+        .fold(
+            (Some(0), Some(0)),
+            |(num_rows, total_byte_size), (rows, bytes)| {
+                (
+                    num_rows.zip(rows).map(|(a, b)| a + b as usize),
+                    total_byte_size.zip(bytes).map(|(a, b)| a + b as usize),
+                )
+            },
+        );
+
+    Ok(Statistics {
+        num_rows: num_rows.map(Precision::Exact).unwrap_or(Precision::Absent),
+        total_byte_size: total_byte_size
+            .map(Precision::Exact)
+            .unwrap_or(Precision::Absent),
+        column_statistics: vec![ColumnStatistics::new_unknown(); num_fields],
+    })
+}
+
 fn stats_for_partitions(
     num_fields: usize,
     partition_stats: impl Iterator<Item = PartitionStats>,
@@ -215,7 +291,6 @@ fn stats_for_partitions(
                 .map(|(a, b)| a + b as usize);
             (num_rows, total_byte_size)
         });
-
     Statistics {
         num_rows: num_rows.map(Precision::Exact).unwrap_or(Precision::Absent),
         total_byte_size: total_byte_size
@@ -291,18 +366,36 @@ impl Stream for AbortableReceiverStream {
             .map_err(|e| ArrowError::ExternalError(Box::new(e)))
     }
 }
+/// Splits the provided partition locations into local and remote partitions.
+/// Local partitions are read directly from local Arrow IPC files,
+/// while remote partitions are fetched using the Arrow Flight client.
+/// If `force_remote_read` is true, all partitions are treated as remote.
+fn local_remote_read_split(
+    partition_locations: Vec<PartitionLocation>,
+    force_remote_read: bool,
+) -> (Vec<PartitionLocation>, Vec<PartitionLocation>) {
+    if !force_remote_read {
+        partition_locations
+            .into_iter()
+            .partition(check_is_local_location)
+    } else {
+        (vec![], partition_locations)
+    }
+}
 
 fn send_fetch_partitions(
     partition_locations: Vec<PartitionLocation>,
     max_request_num: usize,
     max_message_size: usize,
+    force_remote_read: bool,
+    flight_transport: bool,
 ) -> AbortableReceiverStream {
     let (response_sender, response_receiver) = mpsc::channel(max_request_num);
     let semaphore = Arc::new(Semaphore::new(max_request_num));
     let mut spawned_tasks: Vec<SpawnedTask<()>> = vec![];
-    let (local_locations, remote_locations): (Vec<_>, Vec<_>) = partition_locations
-        .into_iter()
-        .partition(check_is_local_location);
+
+    let (local_locations, remote_locations): (Vec<_>, Vec<_>) =
+        local_remote_read_split(partition_locations, force_remote_read);
 
     debug!(
         "local shuffle file counts:{}, remote shuffle file count:{}.",
@@ -315,7 +408,7 @@ fn send_fetch_partitions(
     spawned_tasks.push(SpawnedTask::spawn(async move {
         for p in local_locations {
             let r = PartitionReaderEnum::Local
-                .fetch_partition(&p, max_message_size)
+                .fetch_partition(&p, max_message_size, flight_transport)
                 .await;
             if let Err(e) = response_sender_c.send(r).await {
                 error!("Fail to send response event to the channel due to {e}");
@@ -330,7 +423,7 @@ fn send_fetch_partitions(
             // Block if exceeds max request number.
             let permit = semaphore.acquire_owned().await.unwrap();
             let r = PartitionReaderEnum::FlightRemote
-                .fetch_partition(&p, max_message_size)
+                .fetch_partition(&p, max_message_size, flight_transport)
                 .await;
             // Block if the channel buffer is full.
             if let Err(e) = response_sender.send(r).await {
@@ -356,6 +449,7 @@ trait PartitionReader: Send + Sync + Clone {
         &self,
         location: &PartitionLocation,
         max_message_size: usize,
+        flight_transport: bool,
     ) -> result::Result<SendableRecordBatchStream, BallistaError>;
 }
 
@@ -374,10 +468,11 @@ impl PartitionReader for PartitionReaderEnum {
         &self,
         location: &PartitionLocation,
         max_message_size: usize,
+        flight_transport: bool,
     ) -> result::Result<SendableRecordBatchStream, BallistaError> {
         match self {
             PartitionReaderEnum::FlightRemote => {
-                fetch_partition_remote(location, max_message_size).await
+                fetch_partition_remote(location, max_message_size, flight_transport).await
             }
             PartitionReaderEnum::Local => fetch_partition_local(location).await,
             PartitionReaderEnum::ObjectStoreRemote => {
@@ -390,6 +485,7 @@ impl PartitionReader for PartitionReaderEnum {
 async fn fetch_partition_remote(
     location: &PartitionLocation,
     max_message_size: usize,
+    flight_transport: bool,
 ) -> result::Result<SendableRecordBatchStream, BallistaError> {
     let metadata = &location.executor_meta;
     let partition_id = &location.partition_id;
@@ -411,7 +507,14 @@ async fn fetch_partition_remote(
         })?;
 
     ballista_client
-        .fetch_partition(&metadata.id, partition_id, &location.path, host, port)
+        .fetch_partition(
+            &metadata.id,
+            partition_id,
+            &location.path,
+            host,
+            port,
+            flight_transport,
+        )
         .await
 }
 
@@ -538,6 +641,152 @@ mod tests {
 
         assert_eq!(result, exptected);
     }
+    #[tokio::test]
+    async fn test_stats_for_partition_statistics_no_specific_partition() -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+            Field::new("c", DataType::Int32, false),
+        ]);
+
+        let job_id = "test_job_1";
+        let input_stage_id = 2;
+        let mut partitions: Vec<PartitionLocation> = vec![];
+        for partition_id in 0..4 {
+            partitions.push(PartitionLocation {
+                map_partition_id: 0,
+                partition_id: PartitionId {
+                    job_id: job_id.to_string(),
+                    stage_id: input_stage_id,
+                    partition_id,
+                },
+                executor_meta: ExecutorMetadata {
+                    id: "executor_1".to_string(),
+                    host: "executor_1".to_string(),
+                    port: 7070,
+                    grpc_port: 8080,
+                    specification: ExecutorSpecification { task_slots: 1 },
+                },
+                partition_stats: PartitionStats {
+                    num_rows: Some(1),
+                    num_batches: None,
+                    num_bytes: Some(10),
+                },
+                path: "test_path".to_string(),
+            })
+        }
+
+        let shuffle_reader_exec = ShuffleReaderExec::try_new(
+            input_stage_id,
+            vec![partitions.clone(), partitions],
+            Arc::new(schema),
+            Partitioning::UnknownPartitioning(4),
+        )?;
+
+        let stats = shuffle_reader_exec.partition_statistics(None)?;
+        assert_eq!(8, *stats.num_rows.get_value().unwrap());
+        assert_eq!(80, *stats.total_byte_size.get_value().unwrap());
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_stats_for_partition_statistics_specific_partition() -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+            Field::new("c", DataType::Int32, false),
+        ]);
+
+        let job_id = "test_job_1";
+        let input_stage_id = 2;
+        let mut partitions: Vec<PartitionLocation> = vec![];
+        for partition_id in 0..4 {
+            partitions.push(PartitionLocation {
+                map_partition_id: 0,
+                partition_id: PartitionId {
+                    job_id: job_id.to_string(),
+                    stage_id: input_stage_id,
+                    partition_id,
+                },
+                executor_meta: ExecutorMetadata {
+                    id: "executor_1".to_string(),
+                    host: "executor_1".to_string(),
+                    port: 7070,
+                    grpc_port: 8080,
+                    specification: ExecutorSpecification { task_slots: 1 },
+                },
+                partition_stats: PartitionStats {
+                    num_rows: Some(1),
+                    num_batches: None,
+                    num_bytes: Some(10),
+                },
+                path: "test_path".to_string(),
+            })
+        }
+
+        let shuffle_reader_exec = ShuffleReaderExec::try_new(
+            input_stage_id,
+            vec![partitions.clone(), partitions],
+            Arc::new(schema),
+            Partitioning::UnknownPartitioning(4),
+        )?;
+
+        let stats = shuffle_reader_exec.partition_statistics(Some(3))?;
+        assert_eq!(2, *stats.num_rows.get_value().unwrap());
+        assert_eq!(20, *stats.total_byte_size.get_value().unwrap());
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_stats_for_partition_statistics_specific_partition_out_of_range(
+    ) -> Result<()> {
+        let schema = Schema::new(vec![
+            Field::new("a", DataType::Int32, false),
+            Field::new("b", DataType::Int32, false),
+            Field::new("c", DataType::Int32, false),
+        ]);
+
+        let job_id = "test_job_1";
+        let input_stage_id = 2;
+        let mut partitions: Vec<PartitionLocation> = vec![];
+        for partition_id in 0..4 {
+            partitions.push(PartitionLocation {
+                map_partition_id: 0,
+                partition_id: PartitionId {
+                    job_id: job_id.to_string(),
+                    stage_id: input_stage_id,
+                    partition_id,
+                },
+                executor_meta: ExecutorMetadata {
+                    id: "executor_1".to_string(),
+                    host: "executor_1".to_string(),
+                    port: 7070,
+                    grpc_port: 8080,
+                    specification: ExecutorSpecification { task_slots: 1 },
+                },
+                partition_stats: PartitionStats {
+                    num_rows: Some(1),
+                    num_batches: None,
+                    num_bytes: Some(10),
+                },
+                path: "test_path".to_string(),
+            })
+        }
+
+        let shuffle_reader_exec = ShuffleReaderExec::try_new(
+            input_stage_id,
+            vec![partitions.clone(), partitions],
+            Arc::new(schema),
+            Partitioning::UnknownPartitioning(4),
+        )?;
+
+        let stats = shuffle_reader_exec.partition_statistics(Some(4));
+        assert!(stats.is_err());
+
+        Ok(())
+    }
 
     #[tokio::test]
     async fn test_fetch_partitions_error_mapping() -> Result<()> {
@@ -647,6 +896,36 @@ mod tests {
         }
     }
 
+    // tests if force remote read configuration option will
+    // qualify all partitions as remote
+    #[tokio::test]
+    async fn test_remote_local_read() {
+        let schema = get_test_partition_schema();
+        let data_array = Int32Array::from(vec![1]);
+        let batch =
+            RecordBatch::try_new(Arc::new(schema.clone()), vec![Arc::new(data_array)])
+                .unwrap();
+        let tmp_dir = tempdir().unwrap();
+        let file_path = tmp_dir.path().join("shuffle_data");
+        let file = File::create(&file_path).unwrap();
+        let mut writer = StreamWriter::try_new(file, &schema).unwrap();
+        writer.write(&batch).unwrap();
+        writer.finish().unwrap();
+
+        let partition_locations =
+            get_test_partition_locations(1, file_path.to_str().unwrap().to_string());
+
+        let (local, remote) = local_remote_read_split(partition_locations.clone(), false);
+
+        assert!(!local.is_empty());
+        assert!(remote.is_empty());
+
+        let (local, remote) = local_remote_read_split(partition_locations, true);
+
+        assert!(local.is_empty());
+        assert!(!remote.is_empty());
+    }
+
     async fn test_send_fetch_partitions(max_request_num: usize, partition_num: usize) {
         let schema = get_test_partition_schema();
         let data_array = Int32Array::from(vec![1]);
@@ -665,8 +944,13 @@ mod tests {
             file_path.to_str().unwrap().to_string(),
         );
 
-        let response_receiver =
-            send_fetch_partitions(partition_locations, max_request_num, 4 * 1024 * 1024);
+        let response_receiver = send_fetch_partitions(
+            partition_locations,
+            max_request_num,
+            4 * 1024 * 1024,
+            false,
+            true,
+        );
 
         let stream = RecordBatchStreamAdapter::new(
             Arc::new(schema),
