@@ -17,7 +17,8 @@
 
 use crate::config::{
     BallistaConfig, BALLISTA_GRPC_CLIENT_MAX_MESSAGE_SIZE, BALLISTA_JOB_NAME,
-    BALLISTA_SHUFFLE_READER_MAX_REQUESTS, BALLISTA_STANDALONE_PARALLELISM,
+    BALLISTA_SHUFFLE_READER_FORCE_REMOTE_READ, BALLISTA_SHUFFLE_READER_MAX_REQUESTS,
+    BALLISTA_SHUFFLE_READER_REMOTE_PREFER_FLIGHT, BALLISTA_STANDALONE_PARALLELISM,
 };
 use crate::planner::BallistaQueryPlanner;
 use crate::serde::protobuf::KeyValuePair;
@@ -95,6 +96,21 @@ pub trait SessionConfigExt {
 
     /// Returns parallelism of standalone cluster
     fn ballista_standalone_parallelism(&self) -> usize;
+
+    /// Forces the shuffle reader to always read partitions via the Arrow Flight client,
+    /// even when partitions are local to the node.
+    fn ballista_shuffle_reader_force_remote_read(&self) -> bool;
+    /// Forces the shuffle reader to always read partitions via the Arrow Flight client,
+    /// even when partitions are local to the node.
+    ///
+    /// Enabling forced remote read may significantly reduce performance,
+    /// as all partition reads will go through the Arrow Flight client even for local data.
+    /// Use only when necessary, like in tests
+    fn with_ballista_shuffle_reader_force_remote_read(
+        self,
+        force_remote_read: bool,
+    ) -> Self;
+
     /// Sets parallelism of standalone cluster
     ///
     /// This option to be used to configure standalone session context
@@ -117,6 +133,13 @@ pub trait SessionConfigExt {
         self,
         max_requests: usize,
     ) -> Self;
+
+    fn ballista_shuffle_reader_remote_prefer_flight(&self) -> bool;
+
+    fn with_ballista_shuffle_reader_remote_prefer_flight(
+        self,
+        prefer_flight: bool,
+    ) -> Self;
 }
 
 /// [SessionConfigHelperExt] is set of [SessionConfig] extension methods
@@ -319,6 +342,50 @@ impl SessionConfigExt for SessionConfig {
                 .set_usize(BALLISTA_SHUFFLE_READER_MAX_REQUESTS, max_requests)
         }
     }
+
+    fn ballista_shuffle_reader_force_remote_read(&self) -> bool {
+        self.options()
+            .extensions
+            .get::<BallistaConfig>()
+            .map(|c| c.shuffle_reader_force_remote_read())
+            .unwrap_or_else(|| {
+                BallistaConfig::default().shuffle_reader_force_remote_read()
+            })
+    }
+
+    fn with_ballista_shuffle_reader_force_remote_read(
+        self,
+        force_remote_read: bool,
+    ) -> Self {
+        if self.options().extensions.get::<BallistaConfig>().is_some() {
+            self.set_bool(BALLISTA_SHUFFLE_READER_FORCE_REMOTE_READ, force_remote_read)
+        } else {
+            self.with_option_extension(BallistaConfig::default())
+                .set_bool(BALLISTA_SHUFFLE_READER_FORCE_REMOTE_READ, force_remote_read)
+        }
+    }
+
+    fn ballista_shuffle_reader_remote_prefer_flight(&self) -> bool {
+        self.options()
+            .extensions
+            .get::<BallistaConfig>()
+            .map(|c| c.shuffle_reader_remote_prefer_flight())
+            .unwrap_or_else(|| {
+                BallistaConfig::default().shuffle_reader_remote_prefer_flight()
+            })
+    }
+
+    fn with_ballista_shuffle_reader_remote_prefer_flight(
+        self,
+        prefer_flight: bool,
+    ) -> Self {
+        if self.options().extensions.get::<BallistaConfig>().is_some() {
+            self.set_bool(BALLISTA_SHUFFLE_READER_REMOTE_PREFER_FLIGHT, prefer_flight)
+        } else {
+            self.with_option_extension(BallistaConfig::default())
+                .set_bool(BALLISTA_SHUFFLE_READER_REMOTE_PREFER_FLIGHT, prefer_flight)
+        }
+    }
 }
 
 impl SessionConfigHelperExt for SessionConfig {
@@ -395,7 +462,21 @@ impl SessionConfigHelperExt for SessionConfig {
                 false,
             )
             // same like previous comment
-            .set_bool("datafusion.sql_parser.map_varchar_to_utf8view", false)
+            .set_bool("datafusion.sql_parser.map_string_types_to_utf8view", false)
+            //
+            // As mentioned in https://github.com/apache/datafusion-ballista/issues/1055
+            // "Left/full outer join incorrect for CollectLeft / broadcast"
+            //
+            // In order to make correct results (decreasing performance) CollectLeft
+            // has been disabled until fixed
+            .set_u64(
+                "datafusion.optimizer.hash_join_single_partition_threshold",
+                0,
+            )
+            .set_u64(
+                "datafusion.optimizer.hash_join_single_partition_threshold_rows",
+                0,
+            )
     }
 }
 
