@@ -20,7 +20,7 @@
 //! This module contains a dedicated thread pool for running "cpu
 //! intensive" workloads as query plans
 
-use log::warn;
+use log::{debug, warn};
 use parking_lot::Mutex;
 use std::{pin::Pin, sync::Arc};
 use tokio::sync::oneshot::Receiver;
@@ -84,7 +84,7 @@ impl std::fmt::Debug for DedicatedExecutor {
 }
 
 impl DedicatedExecutor {
-    /// https://stackoverflow.com/questions/62536566
+    /// <https://stackoverflow.com/questions/62536566>
     /// Creates a new `DedicatedExecutor` with a dedicated tokio
     /// runtime that is separate from the `[tokio::main]` threadpool.
     ///
@@ -146,7 +146,7 @@ impl DedicatedExecutor {
         let job = Box::pin(async move {
             let task_output = task.await;
             if tx.send(task_output).is_err() {
-                warn!("Spawned task output ignored: receiver dropped");
+                debug!("Spawned task output ignored: receiver dropped");
             }
         });
 
