@@ -28,7 +28,9 @@ use ballista_core::serde::protobuf::{
 use ballista_core::serde::scheduler::{ExecutorSpecification, PartitionId};
 use ballista_core::serde::BallistaCodec;
 use datafusion::execution::context::TaskContext;
+use datafusion::execution::SessionStateBuilder;
 use datafusion::physical_plan::ExecutionPlan;
+use datafusion::prelude::SessionContext;
 use datafusion_proto::logical_plan::AsLogicalPlan;
 use datafusion_proto::physical_plan::AsExecutionPlan;
 use futures::FutureExt;
@@ -237,6 +239,21 @@ async fn run_received_task<T: 'static + AsLogicalPlan, U: 'static + AsExecutionP
     let task_window_functions = executor.function_registry.window_functions.clone();
 
     let runtime = executor.produce_runtime(&session_config)?;
+
+    // this is temporary fix until we get
+    // https://github.com/apache/datafusion/pull/17601
+    // merged
+    //
+    let session_state = SessionStateBuilder::new()
+        .with_aggregate_functions(task_aggregate_functions.values().cloned().collect())
+        .with_scalar_functions(task_scalar_functions.values().cloned().collect())
+        .with_window_functions(task_window_functions.values().cloned().collect())
+        .with_config(session_config.clone())
+        .with_runtime_env(runtime.clone())
+        .build();
+    let ctx = SessionContext::new_with_state(session_state);
+    //
+
     let session_id = task.session_id.clone();
     let task_context = Arc::new(TaskContext::new(
         Some(task_identity.clone()),
@@ -251,7 +268,7 @@ async fn run_received_task<T: 'static + AsLogicalPlan, U: 'static + AsExecutionP
     let plan: Arc<dyn ExecutionPlan> =
         U::try_decode(task.plan.as_slice()).and_then(|proto| {
             proto.try_into_physical_plan(
-                task_context.deref(),
+                &ctx,
                 runtime.deref(),
                 codec.physical_extension_codec(),
             )
