@@ -46,6 +46,8 @@ use datafusion::physical_plan::ExecutionPlan;
 use std::sync::Arc;
 use tokio::sync::{Mutex, MutexGuard};
 
+use super::{ClusterStateEvent, ClusterStateEventStream};
+
 #[derive(Default)]
 pub struct InMemoryClusterState {
     /// Current available task slots for each executor
@@ -54,6 +56,8 @@ pub struct InMemoryClusterState {
     executors: DashMap<String, ExecutorMetadata>,
     /// Last heartbeat received for each executor
     heartbeats: DashMap<String, ExecutorHeartbeat>,
+
+    cluster_event_sender: ClusterEventSender<ClusterStateEvent>,
 }
 
 impl InMemoryClusterState {
@@ -218,22 +222,57 @@ impl ClusterState for InMemoryClusterState {
         guard.insert(
             executor_id.clone(),
             AvailableTaskSlots {
-                executor_id,
+                executor_id: executor_id.clone(),
                 slots: spec.available_task_slots,
             },
         );
 
+        // RegisteredExecutor event is not pushed from here,
+        // in order to align between push and pull policy
+        // event is pushed from `save_executor_metadata`
+        //
+        // self.cluster_event_sender
+        //     .send(&ClusterStateEvent::RegisteredExecutor {
+        //         executor_id: executor_id.to_string(),
+        //     });
+
         Ok(())
     }
 
     async fn save_executor_metadata(&self, metadata: ExecutorMetadata) -> Result<()> {
-        log::debug!("save executor metadata: {}", metadata.id);
+        log::trace!("save executor metadata: {}", metadata.id);
         // TODO: MM it would make sense to add time when ExecutorMetadata is persisted
         //       we can do that adding additional field in ExecutorMetadata representing
         //       insert time. This information may be useful when reporting executor
         //       status and heartbeat is not available (in case of `TaskSchedulingPolicy::PullStaged`)
-        self.executors.insert(metadata.id.clone(), metadata);
-        Ok(())
+        let executor_id = metadata.id.clone();
+        if self
+            .executors
+            .insert(executor_id.clone(), metadata)
+            .is_none()
+        {
+            self.cluster_event_sender
+                .send(&ClusterStateEvent::RegisteredExecutor {
+                    executor_id: executor_id.to_string(),
+                });
+        }
+
+        //
+        // pull based executor sends `save_executor_metadata`
+        // with all requests, not `ExecutorHeartbeat`, to make it consistent
+        // with push based, registering `ExecutorHeartbeat` every time
+        //
+        let heartbeat = ExecutorHeartbeat {
+            executor_id,
+            timestamp: std::time::SystemTime::now()
+                .duration_since(std::time::UNIX_EPOCH)
+                .expect("Time went backwards")
+                .as_secs(),
+            metrics: vec![],
+            status: None,
+        };
+        self.save_executor_heartbeat(heartbeat).await
+        // Ok(())
     }
 
     async fn get_executor_metadata(&self, executor_id: &str) -> Result<ExecutorMetadata> {
@@ -248,7 +287,7 @@ impl ClusterState for InMemoryClusterState {
     }
 
     async fn save_executor_heartbeat(&self, heartbeat: ExecutorHeartbeat) -> Result<()> {
-        log::debug!("saving executor heartbeat: {}", heartbeat.executor_id);
+        log::trace!("saving executor heartbeat: {}", heartbeat.executor_id);
         let executor_id = heartbeat.executor_id.clone();
         if let Some(mut last) = self.heartbeats.get_mut(&executor_id) {
             let _ = std::mem::replace(last.deref_mut(), heartbeat);
@@ -269,6 +308,11 @@ impl ClusterState for InMemoryClusterState {
         self.executors.remove(executor_id);
         self.heartbeats.remove(executor_id);
 
+        self.cluster_event_sender
+            .send(&ClusterStateEvent::RemovedExecutor {
+                executor_id: executor_id.to_string(),
+            });
+
         Ok(())
     }
 
@@ -286,6 +330,10 @@ impl ClusterState for InMemoryClusterState {
     async fn registered_executor_metadata(&self) -> Vec<ExecutorMetadata> {
         self.executors.iter().map(|v| v.clone()).collect()
     }
+
+    async fn cluster_state_events(&self) -> Result<ClusterStateEventStream> {
+        Ok(Box::pin(self.cluster_event_sender.subscribe()))
+    }
 }
 
 /// Implementation of `JobState` which keeps all state in memory. If using `InMemoryJobState`
@@ -298,8 +346,6 @@ pub struct InMemoryJobState {
     queued_jobs: DashMap<String, (String, u64)>,
     /// In-memory store of running job statuses. Map from Job ID -> JobStatus
     running_jobs: DashMap<String, JobStatus>,
-    /// Active ballista sessions
-    sessions: DashMap<String, Arc<SessionContext>>,
     /// `SessionBuilder` for building DataFusion `SessionContext` from `BallistaConfig`
     session_builder: SessionBuilder,
     /// Sender of job events
@@ -319,7 +365,7 @@ impl InMemoryJobState {
             completed_jobs: Default::default(),
             queued_jobs: Default::default(),
             running_jobs: Default::default(),
-            sessions: Default::default(),
+            //sessions: Default::default(),
             session_builder,
             job_event_sender: ClusterEventSender::new(100),
             config_producer,
@@ -412,42 +458,27 @@ impl JobState for InMemoryJobState {
         Ok(())
     }
 
-    async fn get_session(&self, session_id: &str) -> Result<Arc<SessionContext>> {
-        self.sessions
-            .get(session_id)
-            .map(|session_ctx| session_ctx.clone())
-            .ok_or_else(|| {
-                BallistaError::General(format!("No session for {session_id} found"))
-            })
-    }
-
-    async fn create_session(
-        &self,
-        config: &SessionConfig,
-    ) -> Result<Arc<SessionContext>> {
-        let session = create_datafusion_context(config, self.session_builder.clone())?;
-        self.sessions.insert(session.session_id(), session.clone());
-
-        Ok(session)
-    }
-
-    async fn update_session(
+    async fn create_or_update_session(
         &self,
         session_id: &str,
         config: &SessionConfig,
     ) -> Result<Arc<SessionContext>> {
-        let session = create_datafusion_context(config, self.session_builder.clone())?;
-        self.sessions
-            .insert(session_id.to_string(), session.clone());
+        self.job_event_sender.send(&JobStateEvent::SessionAccessed {
+            session_id: session_id.to_string(),
+        });
 
-        Ok(session)
+        Ok(create_datafusion_context(
+            config,
+            self.session_builder.clone(),
+        )?)
     }
 
-    async fn remove_session(
-        &self,
-        session_id: &str,
-    ) -> Result<Option<Arc<SessionContext>>> {
-        Ok(self.sessions.remove(session_id).map(|(_key, value)| value))
+    async fn remove_session(&self, session_id: &str) -> Result<()> {
+        self.job_event_sender.send(&JobStateEvent::SessionRemoved {
+            session_id: session_id.to_string(),
+        });
+
+        Ok(())
     }
 
     async fn job_state_events(&self) -> Result<JobStateEventStream> {
@@ -516,15 +547,17 @@ impl JobState for InMemoryJobState {
 mod test {
     use std::sync::Arc;
 
-    use crate::cluster::memory::InMemoryJobState;
+    use crate::cluster::memory::{InMemoryClusterState, InMemoryJobState};
     use crate::cluster::test_util::{test_job_lifecycle, test_job_planning_failure};
-    use crate::cluster::{JobState, JobStateEvent};
+    use crate::cluster::{ClusterState, ClusterStateEvent, JobState, JobStateEvent};
     use crate::test_utils::{
         test_aggregation_plan, test_join_plan, test_two_aggregations_plan,
     };
     use ballista_core::error::Result;
     use ballista_core::serde::protobuf::JobStatus;
+    use ballista_core::serde::scheduler::{ExecutorMetadata, ExecutorSpecification};
     use ballista_core::utils::{default_config_producer, default_session_builder};
+    use datafusion::prelude::SessionConfig;
     use futures::StreamExt;
     use tokio::sync::Barrier;
 
@@ -631,4 +664,91 @@ mod test {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_in_memory_cluster_notification() -> Result<()> {
+        let cluster_state = InMemoryClusterState::default();
+
+        let mut event_stream = cluster_state.cluster_state_events().await?;
+
+        let metadata = ExecutorMetadata {
+            id: "id123".to_string(),
+            host: "".to_string(),
+            port: 50055,
+            grpc_port: 50050,
+            specification: ExecutorSpecification { task_slots: 2 },
+        };
+
+        cluster_state
+            .save_executor_metadata(metadata.clone())
+            .await?;
+        let event = event_stream.next().await;
+
+        assert!(matches!(
+            event,
+            Some(
+                ClusterStateEvent::RegisteredExecutor {
+                executor_id
+            }) if executor_id == *"id123",
+
+        ));
+
+        // event should not be emitted as executor is already registered
+        cluster_state
+            .save_executor_metadata(metadata.clone())
+            .await?;
+
+        cluster_state.remove_executor("id123").await?;
+        let event = event_stream.next().await;
+
+        assert!(matches!(
+            event,
+            Some(
+                ClusterStateEvent::RemovedExecutor {
+                executor_id
+            }) if executor_id == *"id123",
+
+        ));
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn test_in_memory_session_notification() -> Result<()> {
+        let state = InMemoryJobState::new(
+            "",
+            Arc::new(default_session_builder),
+            Arc::new(default_config_producer),
+        );
+
+        let event_stream = state.job_state_events().await?;
+
+        state
+            .create_or_update_session("session_id_0", &SessionConfig::new())
+            .await?;
+
+        state.remove_session("session_id_0").await?;
+
+        state
+            .create_or_update_session("session_id_1", &SessionConfig::new())
+            .await?;
+
+        let result = event_stream.take(3).collect::<Vec<JobStateEvent>>().await;
+        assert_eq!(3, result.len());
+        let expected = vec![
+            JobStateEvent::SessionAccessed {
+                session_id: "session_id_0".to_string(),
+            },
+            JobStateEvent::SessionRemoved {
+                session_id: "session_id_0".to_string(),
+            },
+            JobStateEvent::SessionAccessed {
+                session_id: "session_id_1".to_string(),
+            },
+        ];
+
+        assert_eq!(expected, result);
+
+        Ok(())
+    }
 }
