@@ -104,7 +104,7 @@ impl<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan> SchedulerGrpc
                 })?;
 
             let mut available_slots = [AvailableTaskSlots {
-                executor_id,
+                executor_id: executor_id.clone(),
                 slots: num_free_slots,
             }];
             let available_slots = available_slots.iter_mut().collect();
@@ -130,14 +130,28 @@ impl<T: 'static + AsLogicalPlan, U: 'static + AsExecutionPlan> SchedulerGrpc
 
             let mut tasks = vec![];
             for (_, task) in schedulable_tasks {
+                let job_id = task.partition.job_id.clone();
                 match self.state.task_manager.prepare_task_definition(task) {
                     Ok(task_definition) => tasks.push(task_definition),
                     Err(e) => {
                         error!("Error preparing task definition: {e:?}");
+                        if let Err(e) = self.fail_job(job_id, e.to_string()).await {
+                            error!("Error when failing job: {e:?}")
+                        }
                     }
                 }
             }
-            Ok(Response::new(PollWorkResult { tasks }))
+            let jobs_to_clean = self
+                .state
+                .executor_manager
+                .drain_pending_cleanup_jobs(&executor_id)
+                .into_iter()
+                .map(|job_id| CleanJobDataParams { job_id })
+                .collect();
+            Ok(Response::new(PollWorkResult {
+                tasks,
+                jobs_to_clean,
+            }))
         } else {
             warn!("Received invalid executor poll_work request");
             Err(Status::invalid_argument("Missing metadata in request"))
