@@ -27,13 +27,16 @@ use crate::config::SchedulerConfig;
 
 use crate::state::execution_graph::RunningTaskInfo;
 use crate::state::task_manager::JobInfoCache;
+use ballista_core::extension::SessionConfigExt;
 use ballista_core::serde::protobuf::executor_grpc_client::ExecutorGrpcClient;
 use ballista_core::serde::protobuf::{
     executor_status, CancelTasksParams, ExecutorHeartbeat, MultiTaskDefinition,
     RemoveJobDataParams, StopExecutorParams,
 };
 use ballista_core::serde::scheduler::{ExecutorData, ExecutorMetadata};
-use ballista_core::utils::{create_grpc_client_connection, get_time_before};
+use ballista_core::utils::{
+    create_grpc_client_connection, get_time_before, GrpcClientConfig,
+};
 use dashmap::DashMap;
 use log::{debug, error, info, warn};
 use std::collections::{HashMap, HashSet};
@@ -48,6 +51,7 @@ pub struct ExecutorManager {
     config: Arc<SchedulerConfig>,
     clients: ExecutorClients,
     pending_cleanup_jobs: Arc<DashMap<String, HashSet<String>>>,
+    grpc_client_config: GrpcClientConfig,
 }
 
 impl ExecutorManager {
@@ -55,11 +59,20 @@ impl ExecutorManager {
         cluster_state: Arc<dyn ClusterState>,
         config: Arc<SchedulerConfig>,
     ) -> Self {
+        let grpc_client_config =
+            if let Some(config_producer) = &config.override_config_producer {
+                let session_config = config_producer();
+                let ballista_config = session_config.ballista_config();
+                GrpcClientConfig::from(&ballista_config)
+            } else {
+                GrpcClientConfig::default()
+            };
         Self {
             cluster_state,
             config,
             clients: Default::default(),
             pending_cleanup_jobs: Default::default(),
+            grpc_client_config,
         }
     }
 
@@ -70,7 +83,7 @@ impl ExecutorManager {
     }
 
     ///
-    /// Bind the ready to run tasks from [`active_jobs`] to available executors.
+    /// Bind the ready to run tasks from `active_jobs` to available executors.
     pub async fn bind_schedulable_tasks(
         &self,
         running_jobs: Arc<HashMap<String, JobInfoCache>>,
@@ -117,7 +130,10 @@ impl ExecutorManager {
         let executor_manager = self.clone();
         tokio::spawn(async move {
             for (executor_id, infos) in tasks_to_cancel {
-                if let Ok(mut client) = executor_manager.get_client(&executor_id).await {
+                if let Ok(mut client) = executor_manager
+                    .get_client(&executor_id, &executor_manager.grpc_client_config)
+                    .await
+                {
                     if let Err(e) = client
                         .cancel_tasks(CancelTasksParams { task_infos: infos })
                         .await
@@ -174,7 +190,9 @@ impl ExecutorManager {
             let job_id_clone = job_id.to_owned();
 
             if self.config.is_push_staged_scheduling() {
-                if let Ok(mut client) = self.get_client(&executor).await {
+                if let Ok(mut client) =
+                    self.get_client(&executor, &self.grpc_client_config).await
+                {
                     tokio::spawn(async move {
                         if let Err(err) = client
                             .remove_job_data(RemoveJobDataParams {
@@ -226,7 +244,7 @@ impl ExecutorManager {
 
     /// It's only used for pull-based task scheduling.
     ///
-    /// For push-based one, we should use [`register_executor`], instead.
+    /// For push-based one, we should use [`Self::register_executor`], instead.
     pub async fn save_executor_metadata(&self, metadata: ExecutorMetadata) -> Result<()> {
         trace!(
             "save executor metadata {} with {} task slots (pull-based registration)",
@@ -272,7 +290,10 @@ impl ExecutorManager {
 
     pub async fn stop_executor(&self, executor_id: &str, stop_reason: String) {
         let executor_id = executor_id.to_string();
-        match self.get_client(&executor_id).await {
+        match self
+            .get_client(&executor_id, &self.grpc_client_config)
+            .await
+        {
             Ok(mut client) => {
                 tokio::task::spawn(async move {
                     match client
@@ -304,7 +325,9 @@ impl ExecutorManager {
         multi_tasks: Vec<MultiTaskDefinition>,
         scheduler_id: String,
     ) -> Result<()> {
-        let mut client = self.get_client(executor_id).await?;
+        let mut client = self
+            .get_client(executor_id, &self.grpc_client_config)
+            .await?;
         client
             .launch_multi_task(protobuf::LaunchMultiTaskParams {
                 multi_tasks,
@@ -416,7 +439,11 @@ impl ExecutorManager {
             .collect::<Vec<_>>()
     }
 
-    async fn get_client(&self, executor_id: &str) -> Result<ExecutorGrpcClient<Channel>> {
+    async fn get_client(
+        &self,
+        executor_id: &str,
+        grpc_client_config: &GrpcClientConfig,
+    ) -> Result<ExecutorGrpcClient<Channel>> {
         let client = self.clients.get(executor_id).map(|value| value.clone());
 
         if let Some(client) = client {
@@ -427,7 +454,8 @@ impl ExecutorManager {
                 "http://{}:{}",
                 executor_metadata.host, executor_metadata.grpc_port
             );
-            let connection = create_grpc_client_connection(executor_url).await?;
+            let connection =
+                create_grpc_client_connection(executor_url, grpc_client_config).await?;
             let client = ExecutorGrpcClient::new(connection);
 
             {
