@@ -790,8 +790,8 @@ pub fn revive_graph_and_complete_next_stage_with_executor(
     // find the num_available_tasks of the next running stage
     let num_available_tasks = graph
         .stages()
-        .iter()
-        .map(|(_stage_id, stage)| {
+        .values()
+        .map(|stage| {
             if let ExecutionStage::Running(stage) = stage {
                 stage
                     .task_infos
